.filepicker-ui button,
.filepicker-ui input {
    border: 0px;
    font-size: 15px;
    padding: 4px 4px;
    height: 30px;
}

.filepicker-ui button {
    background: var(--default-button-bg);
    color: var(--button-fg);
    /* border-top-left-radius: 0px;
    border-bottom-left-radius: 0px; */
    border: 1px solid;
    border-radius: 4px;
}

.filepicker-ui input {
    border-top-right-radius: 0px;
    border-bottom-right-radius: 0px;
    width: calc(100% - 130px);
}

.filepicker-preview>div {
    border: 1px solid lightgray;
    padding: 5px;
}